#!/usr/bin/perl 

use CallWeaver::Manager;
use strict;
use warnings;
use diagnostics;

$SIG{INT} = \&safe_exit;
$SIG{QUIT} = \&safe_exit;
$SIG{TERM} = \&safe_exit;

# Config vars
#
my $ast_hostname = 'localhost';
my $ast_username = 'webmin';
my $ast_password = 'webmin';

my $astman = new CallWeaver::Manager;
$astman->user($ast_username);
$astman->secret($ast_password);
$astman->host($ast_hostname);

sub printlog($$) {
	my ($a,$b) = @_;
	print "[$a] $b\n";
}


# Enter Loop
my $ast_connected = 1;
while( 1 ) {
	printlog('info', 'Connecting to CallWeaver...');
	if( $astman->connect ) {
		$ast_connected = 1;
		printlog('info', 'Connected to CallWeaver!');
		$astman->setcallback('DEFAULT', \&status_callback);
		eval { $astman->eventloop; };
		exit;
	} else {
		printlog('err', 'Could not connect to CallWeaver!') if $ast_connected;
		$ast_connected = 0;
	}
	sleep 1;
}

#
# Globals
#
my %channels;



# Install signall handler
#

sub Status2String ($) {
	my ($status) = @_;
	#print "STATUS=". $status. ".\n";
	return 'UNDEFINED' 		if (!defined($status));
	return 'Removed'		if ($status eq '-2');
	return 'Deactivated'		if ($status eq '-1');
	return 'Idle'	if ($status eq '0');
	return 'In Use' 		if ($status eq '1');
	return 'Busy' 			if ($status eq '2');
	return 'Unavailable/Unregistered' 		if ($status eq '4');
	return 'Ringing' 		if ($status eq '8');
	return 'Call Waiting???'	if ($status eq '9');
	return 'On Hold???'		if ($status eq '16');
	return 'UNKNOWN:'. $status.".";
}

sub DeviceState2String ($) {
	my ($status) = @_;
	return 'UNDEFINED' 	if (!defined($status));
	return 'Unknown'	if ($status eq '0');
	return 'Idle'		if ($status eq '1');
	return 'In Use'		if ($status eq '2');
	return 'Busy' 		if ($status eq '3');
	return 'Invalid' 	if ($status eq '4');
	return 'Unavailable' 	if ($status eq '5');
	return 'Ringing' 	if ($status eq '6');
	return 'DeviState2String('. $status.")";
}



sub status_callback {
	my (%E) = @_;

	#
	# Disply Events
	#
	for ($E{Event}) {

		/PeerStatus/ && do {
			print "\t". $E{Peer}. " ". $E{PeerStatus}. "\n";
			return;
		};
		/ExtensionStatus/ && do {
			print "\t". $E{Exten}. " @ ". $E{Context}. " is ". Status2String($E{Status}). "\n";
			return;
		};
		/QueueMemberStatus/ && do {
			print "\t". $E{Location}. " is ". DeviceState2String($E{Status}). " in queue ". $E{Queue}. "\n";
			return;
		};
		/Registry/ && do {
			print "\t". $E{Channel}. "/". $E{Domain}. " has ". $E{Status}. "\n";
			return;
		};
		/Newchannel/ && do {
			$E{Status} = 'UNDEFINED' if (!defined($E{Status}));
			print "\t\tNChan: ".  $E{CallerIDName}. ' <'.  $E{CallerID}. '> is '.  $E{Status}. "\n";
			return;
		};
		/Newcallerid/ && do {
			$E{Status} = 'UNDEFINED' if (!defined($E{Status}));
			print "\t\tNewCLI: ".  $E{CallerIDName}. ' <'.  $E{CallerID}. '> on '.  $E{Channel}. "\n";
			return;
		};
		/Dial/ && do {
			print "\t\t". "Dial\t". $E{CallerIDName}. ' <'. $E{CallerID}. '>'. 
				"\t". $E{Source}. "=>". $E{Destination}. "\t".
				$E{SrcUniqueID}. "=>". $E{DestUniqueID}. "\n";
			return;
		};
		/Hangup/ && do {
			print "\t\tHangup ". $E{Channel}. "\t". $E{Cause}. " ". $E{'Cause-txt'}. "\n";
			return;
		};
		/Newexten/ && do {
			print "exten => ". $E{Context}. ", ". $E{Extension}. ", ". $E{Priority}. ", ".
				$E{Application}. " ( ". $E{AppData}. " )\n";
			return;
		};
		/^UserEvent/ && do {
			my $event_name = $E{Event};
			$event_name =~ s/^UserEvent//;
			$E{Source} = 'UNDEFINED' if (!defined($E{Source}));
			print "[". $event_name. "]". " ". $E{Source}. "\n";
			return;
		};
		/Cdr/ && do {
			print "CDR: ". $E{Source}. '=>'. $E{Destination}. "\t".
				$E{Disposition}. "\t". $E{Duration}. "\n";
			return;
		};
		/Shutdown/ && do {
			print "Shutdown: ". $E{Shutdown}. "\tRestart: ". $E{Restart}. "\n";
			return;
		};
	}

	#
	# DEBUG the whole event out
	#
	# TODO: put a flag here for display on/off
	# TODO: put a flag here for display horizontal/vertical
	print "> ";
	return unless %E;
	foreach (keys %E) {
		next if ($_ eq 'Privilege');
		print "$_: ". $E{$_}. ", ";
	}
	print "\n";

	#
	# ERROR, Unhandled event
	#
	print "[UNKNOW] Event: ". $E{Event}. "\n";
	return;
}


# Signal Handlers
#
sub safe_exit {
	my($sig) = @_;
	printlog('crit', "Caught a SIG$sig - shutting down");
	$astman->disconnect if $ast_connected;     
	exit;
}

